@@ -446,7 +446,7 @@ def get_test_metadata(test_id, session=None):
     """
     session = session or get_session()
     query = db_utils.model_query(models.TestMetadata, session).filter_by(
-        id=test_id)
+        test_id=test_id)
     return query.all()
 
 
@@ -672,12 +672,12 @@ def get_latest_run(session=None):
 
 
 def get_failing_from_run(run_id, session=None):
-    """Return the set of failing test_ids for a give run.
+    """Return the set of failing test runs for a give run.
 
     This method will return all the test run objects that failed during the
     specified run.
 
-    :param str run_id: uuid for run tho find all the failing runs
+    :param str run_id: uuid for the run to find all the failing runs
     :param session: optional session object if one isn't provided a new session
                     will be acquired for the duration of this operation
 
@@ -1522,6 +1522,31 @@ def add_test_run_attachments(attach_dict, test_run_id, session=None):
     return attachments
 
 
+def get_recent_failed_runs_by_run_metadata(key, value, num_runs=10,
+                                           start_date=None, session=None):
+    """Get a list of recent failed runs for a given run metadata pair
+
+    :param str key: the run_metadata key to get failed runs
+    :param str value: The run_metadata value to get failed runs
+    :param int num_runs: The number of results to fetch, defaults to 10
+    :param datetime start_date: the optional starting dates to get runs from.
+                                Nothing older than this date will be returned
+    :param session: optional session object if one isn't provided a new session
+                    will be acquired for the duration of this operation
+
+    :return list: The list of recent failed Run objects
+    :rtype: subunit2sql.models.Run
+    """
+    session = session or get_session()
+    query = db_utils.model_query(models.Run, session).join(
+        models.RunMetadata, models.Run.id == models.RunMetadata.run_id).filter(
+            models.RunMetadata.key == key,
+            models.RunMetadata.value == value)
+    query = _filter_runs_by_date(query, start_date)
+    return query.filter(models.Run.fails > 0).order_by(
+        models.Run.run_at.desc()).limit(num_runs).all()
+
+
 def get_runs_by_status_grouped_by_run_metadata(key, start_date=None,
                                                stop_date=None, session=None):
     session = session or get_session()
