@@ -36,9 +36,9 @@ class TestDatabaseAPI(base.TestCase):
 
     def setUp(self):
         super(TestDatabaseAPI, self).setUp()
+        self.useFixture(fixtures.LockFixture(self.dialect))
         if not db_test_utils.is_backend_avail(self.dialect):
             raise self.skipTest('%s is not available' % self.dialect)
-        self.useFixture(fixtures.LockFixture(self.dialect))
         if self.dialect == 'mysql':
             self.useFixture(fixtures.MySQLConfFixture())
         elif self.dialect == 'postgres':
@@ -669,3 +669,149 @@ class TestDatabaseAPI(base.TestCase):
         fail_rate = api.get_run_failure_rate_by_key_value_metadata(
             'a_key', 'a_value')
         self.assertEqual(50, fail_rate)
+
+    def test_get_test_prefixes(self):
+        api.create_test('prefix.token.token')
+        api.create_test('setUpClass (prefix.token.token)')
+        api.create_test('other.token.token')
+        api.create_test('justonetoken')
+
+        prefixes = api.get_test_prefixes()
+        self.assertEqual(len(prefixes), 3)
+        self.assertIn('prefix', prefixes)
+        self.assertIn('other', prefixes)
+        self.assertIn('justonetoken', prefixes)
+
+    def test_get_tests_by_prefix(self):
+        api.create_test('prefix.token.token')
+        api.create_test('setUpClass (prefix.token.token)')
+        api.create_test('other.token.token')
+        api.create_test('justonetoken')
+
+        tests = api.get_tests_by_prefix('prefix')
+        self.assertEqual(len(tests), 2)
+        self.assertIn('prefix.token.token', [test.test_id for test in tests])
+        self.assertIn('setUpClass (prefix.token.token)',
+                      [test.test_id for test in tests])
+
+        tests = api.get_tests_by_prefix('other')
+        self.assertEqual(len(tests), 1)
+        self.assertIn('other.token.token', [test.test_id for test in tests])
+
+        tests = api.get_tests_by_prefix('prefix', limit=1, offset=1)
+        self.assertEqual(len(tests), 1)
+        self.assertIn('setUpClass (prefix.token.token)',
+                      [test.test_id for test in tests])
+
+        tests = api.get_tests_by_prefix('justonetoken')
+        self.assertEqual(len(tests), 1)
+        self.assertIn('justonetoken', [test.test_id for test in tests])
+
+    def test_update_test(self):
+        create_res = api.create_test('fake_test')
+        values = {'run_count': 2, 'run_time': 1.2}
+        update_res = api.update_test(values, create_res.id)
+        res = api.get_test_by_test_id('fake_test')
+        self.assertEqual(res.id, update_res.id)
+        self.assertEqual(res.test_id, 'fake_test')
+        self.assertEqual(res.run_time, 1.2)
+        self.assertEqual(res.run_count, 2)
+
+    def test_update_test_run(self):
+        run = api.create_run()
+        test = api.create_test('fake_test')
+        test_run = api.create_test_run(test.id, run.id, 'fail')
+        start_time = datetime.datetime.utcnow()
+        stop_time = datetime.datetime.utcnow()
+        start_time = start_time.replace(microsecond=0)
+        stop_time = start_time.replace(microsecond=0)
+        values = {'status': 'success', 'start_time': start_time,
+                  'stop_time': stop_time}
+        update_test_run = api.update_test_run(values, test_run.id)
+        all_test_runs = api.get_all_test_runs()
+        self.assertEqual(len(all_test_runs), 1)
+        self.assertEqual(update_test_run.id, all_test_runs[0].id)
+        self.assertEqual(all_test_runs[0].status, 'success')
+        self.assertEqual(all_test_runs[0].start_time, start_time)
+        self.assertEqual(all_test_runs[0].stop_time, stop_time)
+
+    def test_get_test_run_by_id(self):
+        run = api.create_run()
+        test = api.create_test('fake_test')
+        test_run = api.create_test_run(test.id, run.id, 'fail')
+        res = api.get_test_run_by_id(test_run.id)
+        self.assertEqual(res.id, test_run.id)
+        self.assertEqual(res.status, 'fail')
+        self.assertEqual(res.run_id, run.id)
+
+    def test_get_test_by_id(self):
+        create_res = api.create_test('fake_test', 2, 1, 1, 1.2)
+        res = api.get_test_by_id(create_res.id)
+        self.assertEqual(res.id, create_res.id)
+        self.assertEqual(res.test_id, 'fake_test')
+        self.assertEqual(res.run_time, 1.2)
+        self.assertEqual(res.run_count, 2)
+
+    def test_get_run_id_from_uuid(self):
+        run = api.create_run()
+        run_id = api.get_run_id_from_uuid(run.uuid)
+        self.assertEqual(run_id, run.id)
+
+    def test_get_run_by_id(self):
+        run = api.create_run()
+        res = api.get_run_by_id(run.id)
+        self.assertEqual(res.id, run.id)
+        self.assertEqual(res.uuid, run.uuid)
+
+    def test_get_test_runs_by_test_id(self):
+        run_a = api.create_run()
+        run_b = api.create_run()
+        run_c = api.create_run()
+        test = api.create_test('fake_test')
+        api.create_test_run(test.id, run_a.id, 'success')
+        api.create_test_run(test.id, run_b.id, 'fail')
+        api.create_test_run(test.id, run_c.id, 'success')
+        res = api.get_test_runs_by_test_id(test.id)
+        self.assertEqual(3, len(res))
+        self.assertIn(run_a.id, [x.run_id for x in res])
+        self.assertIn(run_b.id, [x.run_id for x in res])
+        self.assertIn(run_c.id, [x.run_id for x in res])
+
+    def test_get_test_metadata(self):
+        test = api.create_test('fake_test')
+        test_meta = {
+            'test_a': 'a',
+            'test_b': 'b',
+            'test_c': 'c',
+        }
+        api.add_test_metadata(test_meta, test.id)
+        test_metadata = api.get_test_metadata(test.id)
+        self.assertEqual(3, len(test_metadata))
+        for meta in test_metadata:
+            self.assertIn(meta.key, test_meta.keys())
+            self.assertIn(meta.value, test_meta.values())
+
+    def test_get_recent_failed_runs_by_run_metadata_no_start_date(self):
+        run_a = api.create_run(fails=1)
+        api.create_run()
+        run_c = api.create_run(fails=2)
+        api.add_run_metadata({'fake_key': 'fake_value'}, run_a.id)
+        api.add_run_metadata({'zeon': 'zaku'}, run_c.id)
+        results = api.get_recent_failed_runs_by_run_metadata('zeon', 'zaku')
+        self.assertEqual(1, len(results))
+        self.assertEqual(run_c.id, results[0].id)
+
+    def test_get_recent_failed_runs_by_run_metadata_with_start_date(self):
+        run_a = api.create_run(fails=1)
+        api.create_run()
+        run_c = api.create_run(fails=2)
+        run_d = api.create_run(fails=3,
+                               run_at=datetime.datetime(1914, 6, 28,
+                                                        10, 45, 0))
+        api.add_run_metadata({'fake_key': 'fake_value'}, run_a.id)
+        api.add_run_metadata({'zeon': 'zaku'}, run_c.id)
+        api.add_run_metadata({'zeon': 'zaku'}, run_d.id)
+        results = api.get_recent_failed_runs_by_run_metadata(
+            'zeon', 'zaku', start_date=datetime.date(1970, 1, 1))
+        self.assertEqual(1, len(results))
+        self.assertEqual(run_c.id, results[0].id)
