@@ -14,13 +14,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import ConfigParser
 import datetime
 import os
-
+import uuid
 
 from alembic import config
 from alembic import script
+import six
+from six.moves import configparser as ConfigParser
 import sqlalchemy
 from sqlalchemy.engine import reflection
 
@@ -137,10 +138,10 @@ class TestWalkMigrations(base.TestCase):
             self.fail("Shouldn't have connected")
 
     def test_mysql_opportunistically(self):
+        self.useFixture(fixtures.LockFixture('mysql'))
         if not db_test_utils.is_backend_avail('mysql'):
             raise self.skipTest('mysql is not available')
 
-        self.useFixture(fixtures.LockFixture('mysql'))
         self.useFixture(fixtures.MySQLConfFixture())
         # Test that table creation on mysql only builds InnoDB tables
         # add this to the global lists to make reset work with it, it's removed
@@ -181,9 +182,9 @@ class TestWalkMigrations(base.TestCase):
 
     def test_postgresql_opportunistically(self):
         # Test postgresql database migration walk
+        self.useFixture(fixtures.LockFixture('postgres'))
         if not db_test_utils.is_backend_avail('postgres'):
             raise self.skipTest('postgres is not available')
-        self.useFixture(fixtures.LockFixture('postgres'))
         self.useFixture(fixtures.PostgresConfFixture())
         # add this to the global lists to make reset work with it, it's removed
         # automatically in tearDown so no need to clean it up here.
@@ -274,7 +275,7 @@ class TestWalkMigrations(base.TestCase):
         runs.insert().values(time_data).execute()
         runs = get_table(engine, 'runs')
         result = runs.select().execute()
-        run_at = map(lambda x: (x['id'], x['run_at']), result)
+        run_at = list(map(lambda x: (x['id'], x['run_at']), result))
         for run in data:
             self.assertIn((run['id'], None), run_at)
         self.assertIn((time_data['id'], now), run_at)
@@ -321,7 +322,7 @@ class TestWalkMigrations(base.TestCase):
         # Query the DB for the tests from the sample dataset above
         where = ' OR '.join(["tests.id='%s'" % x for x in test_ids])
         result = tests.select(where).execute()
-        run_time_pairs = map(lambda x: (x['id'], x['run_time']), result)
+        run_time_pairs = list(map(lambda x: (x['id'], x['run_time']), result))
         # Ensure the test with one failure is None
         self.assertIn(('fake_null_test_id_fails', None), run_time_pairs)
         # Ensure the test with 2 success each taking 4 sec lists the proper
@@ -398,3 +399,176 @@ class TestWalkMigrations(base.TestCase):
         runs = [indx for indx in indxs if indx['column_names'] == ['run_id']]
         self.assertEqual(1, len(tests))
         self.assertEqual(1, len(runs))
+
+    def _pre_upgrade_2822a408bdd0(self, engine):
+        data = {}
+
+        # Add run
+        runs = get_table(engine, 'runs')
+        run = {'id': six.text_type(uuid.uuid4()),
+               'skips': 0,
+               'fails': 0,
+               'passes': 1,
+               'run_time': 1.0,
+               'artifacts': 'https://am_i_really_a_fake_url',
+               'run_at': datetime.datetime.utcnow()}
+        runs.insert().values(run).execute()
+        data['run'] = run
+        # Add test_metadata
+        run_metadatas = get_table(engine, 'run_metadata')
+        run_metadata = {'id': six.text_type(uuid.uuid4()),
+                        'run_id': run['id'],
+                        'key': 'attrs',
+                        'value': 'an_attr'}
+        run_metadatas.insert().values(run_metadata).execute()
+        data['run_metadata'] = run_metadata
+
+        # Add test
+        tests = get_table(engine, 'tests')
+        test = {'id': six.text_type(uuid.uuid4()),
+                'test_id': 'I_am_a_real_test!',
+                'success': 1,
+                'failure': 0}
+        tests.insert().values(test).execute()
+        data['test'] = test
+
+        # Add test_metadata
+        test_metadatas = get_table(engine, 'test_metadata')
+        test_metadata = {'id': six.text_type(uuid.uuid4()),
+                         'test_id': test['id'],
+                         'key': 'a_real_key',
+                         'value': 'an_attr'}
+        test_metadatas.insert().values(test_metadata).execute()
+        data['test_metadata'] = test_metadata
+
+        # Add test run
+        now = datetime.datetime.now()
+        future_now = now + datetime.timedelta(0, 4)
+
+        test_runs = get_table(engine, 'test_runs')
+
+        test_run = {'id': six.text_type(uuid.uuid4()),
+                    'test_id': test['id'],
+                    'run_id': run['id'],
+                    'start_time': now,
+                    'status': 'success',
+                    'stop_time': future_now}
+        test_runs.insert().values(test_run).execute()
+        data['test_run'] = test_run
+
+        # Add test_run_metadata
+        test_run_metadatas = get_table(engine, 'test_run_metadata')
+        test_run_metadata = {'id': six.text_type(uuid.uuid4()),
+                             'test_run_id': test_run['id'],
+                             'key': 'attrs',
+                             'value': 'an_attr'}
+        test_run_metadatas.insert().values(test_run_metadata).execute()
+        data['test_run_metadata'] = test_run_metadata
+
+        attachments = get_table(engine, 'attachments')
+        attachment = {'id': six.text_type(uuid.uuid4()),
+                      'test_run_id': test_run['id'],
+                      'label': 'an_attachment',
+                      'attachment': b'something'}
+        attachments.insert().values(attachment).execute()
+        data['attachment'] = attachment
+        return data
+
+    def _check_2822a408bdd0(self, engine, data):
+        # Check Primary Keys
+        insp = reflection.Inspector(engine)
+        runs_pk = insp.get_pk_constraint('runs')
+        self.assertEqual(['id'], runs_pk['constrained_columns'])
+        run_meta_pk = insp.get_pk_constraint('run_metadata')
+        self.assertEqual(['id'], run_meta_pk['constrained_columns'])
+        tests_pk = insp.get_pk_constraint('tests')
+        self.assertEqual(['id'], tests_pk['constrained_columns'])
+        test_meta_pk = insp.get_pk_constraint('test_metadata')
+        self.assertEqual(['id'], test_meta_pk['constrained_columns'])
+        test_runs_pk = insp.get_pk_constraint('runs')
+        self.assertEqual(['id'], test_runs_pk['constrained_columns'])
+        test_run_meta_pk = insp.get_pk_constraint('tests')
+        self.assertEqual(['id'], test_run_meta_pk['constrained_columns'])
+        attach_pk = insp.get_pk_constraint('attachments')
+        self.assertEqual(['id'], attach_pk['constrained_columns'])
+
+        if engine.dialect.name == 'sqlite':
+            new_id_type = sqlalchemy.Integer
+        else:
+            new_id_type = sqlalchemy.BigInteger
+
+        # Check id column type
+        runs_col = [x for x in insp.get_columns(
+            'runs') if x['name'] == 'id'][0]
+        self.assertIsInstance(runs_col['type'], new_id_type)
+        run_meta_col = [x for x in insp.get_columns(
+            'run_metadata') if x['name'] == 'id'][0]
+        self.assertIsInstance(run_meta_col['type'], new_id_type)
+        tests_col = [x for x in insp.get_columns(
+            'tests') if x['name'] == 'id'][0]
+        self.assertIsInstance(tests_col['type'], new_id_type)
+        test_meta_col = [x for x in insp.get_columns(
+            'test_metadata') if x['name'] == 'id'][0]
+        self.assertIsInstance(test_meta_col['type'], new_id_type)
+        test_runs_col = [x for x in insp.get_columns(
+            'test_runs') if x['name'] == 'id'][0]
+        self.assertIsInstance(test_runs_col['type'], new_id_type)
+        test_run_meta_col = [x for x in insp.get_columns(
+            'test_run_metadata') if x['name'] == 'id'][0]
+        self.assertIsInstance(test_run_meta_col['type'], new_id_type)
+        attach_col = [x for x in insp.get_columns(
+            'attachments') if x['name'] == 'id'][0]
+        self.assertIsInstance(attach_col['type'], new_id_type)
+
+        # Check all the new ids match
+        runs_t = get_table(engine, 'runs')
+        run_ids = [x[1] for x in runs_t.select().execute()]
+        run_metadatas_t = get_table(engine, 'run_metadata')
+        tests_t = get_table(engine, 'tests')
+        test_metadatas_t = get_table(engine, 'test_metadata')
+        test_runs_t = get_table(engine, 'test_runs')
+        test_runs_raw = list(test_runs_t.select().execute())
+        test_run_test_ids = [x[1] for x in test_runs_raw]
+        test_run_metadatas_t = get_table(engine, 'test_run_metadata')
+        attachments_t = get_table(engine, 'attachments')
+        # Get test we inserted before migration
+        test_row = list(tests_t.select().where(
+            tests_t.c.test_id == data['test']['test_id']).execute())[0]
+        self.assertIn(test_row[0], test_run_test_ids)
+        # Check test run
+        test_run_row = list(test_runs_t.select().where(
+            test_runs_t.c.test_id == test_row[0]).execute())[0]
+        self.assertEqual(test_run_row[3], data['test_run']['status'])
+        self.assertEqual(test_run_row[4].replace(microsecond=0),
+                         data['test_run']['start_time'].replace(microsecond=0))
+        self.assertEqual(test_run_row[5].replace(microsecond=0),
+                         data['test_run']['stop_time'].replace(microsecond=0))
+        self.assertIn(test_run_row[2], run_ids)
+        # Check run
+        run_row = list(runs_t.select().where(
+            runs_t.c.id == test_run_row[2]).execute())[0]
+        self.assertEqual(data['run']['artifacts'], run_row[6])
+        self.assertEqual(data['run']['id'], run_row[0])
+        # Check run metadata
+        run_metadata_row = list(run_metadatas_t.select().where(
+            run_metadatas_t.c.run_id == run_row[1]).execute())[0]
+        self.assertEqual(data['run_metadata']['key'], run_metadata_row[1])
+        self.assertEqual(data['run_metadata']['value'], run_metadata_row[2])
+        # Check test metadata
+        test_metadata_row = list(test_metadatas_t.select().where(
+            test_metadatas_t.c.test_id == test_row[0]).execute())[0]
+        self.assertEqual(data['test_metadata']['key'], test_metadata_row[1])
+        self.assertEqual(data['test_metadata']['value'], test_metadata_row[2])
+        # Check test run metadata
+        test_run_metadata_row = list(test_run_metadatas_t.select().where(
+            test_run_metadatas_t.c.test_run_id == test_run_row[0]).execute())
+        test_run_metadata_row = test_run_metadata_row[0]
+        self.assertEqual(data['test_run_metadata']['key'],
+                         test_run_metadata_row[1])
+        self.assertEqual(data['test_run_metadata']['value'],
+                         test_run_metadata_row[2])
+        # Check attachment
+        attachment_row = list(attachments_t.select().where(
+            attachments_t.c.test_run_id == test_run_row[0]).execute())[0]
+        self.assertEqual(data['attachment']['label'], attachment_row[2])
+        self.assertEqual(data['attachment']['attachment'], attachment_row[3])
